@@ -6,7 +6,7 @@ import collections
 from collections import OrderedDict
 from gridded.utilities import get_dataset, _reorganize_spatial_data,\
     _align_results_to_spatial_data
-from gridded.grids import Grid, Grid_U, Grid_S
+from gridded.grids import Grid, Grid_U, Grid_S, Grid_R
 from gridded.depth import Depth
 from gridded.time import Time
 
@@ -209,9 +209,11 @@ class Variable(object):
             if time_origin is not None:
                 time = Time(data=time.data, filename=time.filename, varname=time.varname, origin=time_origin)
         if depth is None:
-            if (isinstance(grid, Grid_S) and len(data.shape) == 4 or
+            if (isinstance(grid, (Grid_S, Grid_R)) and len(data.shape) == 4 or
                     isinstance(grid, Grid_U) and len(data.shape) == 3):
-                depth = Depth(surface_index=-1)
+                depth = Depth.from_netCDF(grid_file,
+                                          dataset=dg,
+                                          )
 #             if len(data.shape) == 4 or (len(data.shape) == 3 and time is None):
 #                 from gnome.environment.environment_objects import S_Depth
 #                 depth = S_Depth.from_netCDF(grid=grid,
@@ -367,7 +369,7 @@ class Variable(object):
         if self._order is not None:
             return self._order
         else:
-            if isinstance(self.grid, Grid_S):
+            if isinstance(self.grid, (Grid_S, Grid_R)):
                 order = ['time', 'depth', 'lon', 'lat']
             else:
                 order = ['time', 'depth', 'ele']
@@ -568,9 +570,9 @@ class Variable(object):
                 return n
         for n in std_names_list:
             for var in df.variables.values():
-                if hasattr(var, 'standard_name') or hasattr(var, 'long_name'):
-                    if var.name == n:
-                        return n
+                if (hasattr(var, 'standard_name') and var.standard_name == n or
+                        hasattr(var, 'long_name') and var.long_name == n):
+                    return var.name
         raise ValueError("Default names not found.")
 
 
@@ -718,9 +720,11 @@ class VectorVariable(object):
             if time_origin is not None:
                 time = Time(data=time.data, filename=data_file, varname=time.varname, origin=time_origin)
         if depth is None:
-            if (isinstance(grid, Grid_S) and len(data.shape) == 4 or
+            if (isinstance(grid, (Grid_S, Grid_R)) and len(data.shape) == 4 or
                     isinstance(grid, Grid_U) and len(data.shape) == 3):
-                depth = Depth(surface_index=-1)
+                depth = Depth.from_netCDF(grid_file,
+                                          dataset=dg,
+                                          )
 
 #         if depth is None:
 #             if (isinstance(grid, Grid_S) and len(data.shape) == 4 or
@@ -954,8 +958,10 @@ class VectorVariable(object):
                     k = kws
                     s = shared
                     return (n in s) and ((n not in k) or (n in k and k[n] is None))
+
                 if 'filename' in kws and kws['filename'] is not None:
                     kws['data_file'] = kws['grid_file'] = kws['filename']
+                ds = dg =  None
                 if _mod('dataset'):
                     if 'grid_file' in kws and 'data_file' in kws:
                         if kws['grid_file'] == kws['data_file']:
@@ -972,7 +978,7 @@ class VectorVariable(object):
                     ds = kws['dataset']
                 if _mod('grid'):
                     gt = kws.get('grid_topology', None)
-                    kws['grid'] = Grid.from_netCDF(kws['grid_file'], dataset=dg, grid_topology=gt)
+                    kws['grid'] = Grid.from_netCDF(kws['filename'], dataset=dg, grid_topology=gt)
                 if kws.get('varnames', None) is None:
                     varnames = cls._gen_varnames(kws['data_file'],
                                                  dataset=ds)
